@@ -22,7 +22,7 @@
 # mkbackup is released under the GNU GPL license
 #
 
-import sys, os, subprocess, time, smtplib, calendar, re, socket, urllib2
+import sys, os, subprocess, time, smtplib, calendar, re, socket, urllib2, logging, logging.handlers
 
 from optparse import OptionParser
 import ConfigParser
@@ -37,7 +37,7 @@ from datetime import datetime, timedelta
 
 import cron
 
-__version__='0.1.2'
+__version__='0.1.4'
 last_version_url='http://www.magikmon.com/download/mkbackup_last.txt'
 
 atom=r"[a-zA-Z0-9_#\$&'*+/=?\^`{}~|\-]+"
@@ -197,7 +197,7 @@ class NtBackup:
     types=dict(normal='normal', full='normal', incremental='incremental', inc='incremental', differential='differential', diff='differential', copy='copy', daily='daily')
 
     # -----------------------------------------------------------------------
-    def run(self, command, job):
+    def run(self, command, job, log):
 
         errors, warning={}, {}
 
@@ -219,7 +219,7 @@ class NtBackup:
         check_boolean(job, errors, [('verify', 'no'), ('restricted', 'no'), ])
 
         logdir_default=os.path.join(os.environ.get('USERPROFILE'), 'Local Settings\Application Data\Microsoft\Windows NT\NTBackup\data')
-        print 'logdir (default)', logdir_default
+        log.info('logdir_default=%s', logdir_default)
         logdir=job.setdefault('logdir', logdir_default)
         if not os.path.isdir(logdir):
             errors['logdir']='directory not found'
@@ -245,9 +245,10 @@ class NtBackup:
 
         msg_body=''
         if errors:
+            log.error('Error in section: %s', job['name'])
             for k, v in errors.iteritems():
+                log.error('%s=%r %s', k, job.get(k,''), v)
                 msg_body+='%s=%r\n    %s\n\n' % (k, job.get(k,''), v)
-            print 'Error in section: %s\n%s' % (job['name'], msg_body)
             if mail_config and not command=='check':
                 subject='MKBACKUP: CONFIG ERR %s ' % (command, job['name'])
                 sendmail(job['sender'], job['recipients'], subject, msg_body, [], job['smtp_host'], job['smtp_port'])
@@ -255,7 +256,7 @@ class NtBackup:
             # --------->  RETURN <-----------
             return
 
-        print 'No error in section: %s' % job['name']
+        log.info('No error in section: %s', job['name'])
 
         #
         # 
@@ -265,45 +266,60 @@ class NtBackup:
         
         now=datetime.now()
         if command in ('check', 'checkmail'):
-            msg_body+='Destinations by date:\n'
+            msg_body+='Destinations day by day:\n'
+            log.info('Destinations day by day:')
             for i in range(destination.max_weekdivisor*7):
                 today=now+timedelta(days=i)
                 typ, target=destination.match(today)
-                msg_body+='    %s %-12s %s\n' % (today.strftime('%a %d %b %Y'), typ, target)
-                if typ!='exempted' and not os.path.isdir(os.path.dirname(target)):
+                if typ=='none':
+                    ty='none'
+                    target='-----------------'
+                elif typ==None:
+                    ty=''
+                    target='??? no target ???'
+                else:
+                    ty=typ
+                msg_body+='    %s %-12s %s\n' % (today.strftime('%a %d %b %Y'), ty, target)
+                log.info('    %s %-12s %s', today.strftime('%a %d %b %Y'), ty, target)
+                if typ!='none' and typ!=None and not os.path.isdir(os.path.dirname(target)):
                     msg_body+='        directory not found !!!\n'
+                    log.warning('        directory not found: %s', os.path.dirname(target))
 
-        print msg_body
-        
         typ, target=destination.match(now)
         
-        if not command in ('check', 'checkmail') and typ=='exempted':
-            print 'EXEMPTED today'
-            print 'bye'
-            sys.exit(0)
+        if typ=='none':
+            cmdline='no backup today'
+            log.info('no backup today')
+        elif typ==None:
+            cmdline='no target for today !'
+            log.error('no target for today')
+        else:
+            args=[ program_exe, 'backup', job.get('selection'), '/J', job_name, '/M', typ, '/F', target, '/rs:no', ]
+            if job.get('description', None): args.extend(['/d', job.get('description')])
+            args.append('/v:'+self.boolean[job.get('verify')])
+            args.append('/r:'+self.boolean[job.get('restricted')])
+            args.append('/l:s') # logging [s]ummary
+            # '/hc:off'
         
-        args=[ program_exe, 'backup', job.get('selection'), '/J', job_name, '/M', typ, '/F', target, '/rs:no', ]
-        if job.get('description', None): args.extend(['/d', job.get('description')])
-        args.append('/v:'+self.boolean[job.get('verify')])
-        args.append('/r:'+self.boolean[job.get('restricted')])
-        args.append('/l:s') # logging [s]ummary
-        # '/hc:off'
-        
-        cmdline=' '.join(args)
+            cmdline=' '.join(args)
+            log.info('cmdline=%s', cmdline)
+
         msg_body+='\n%s\n' % cmdline
         
-        print cmdline
-        
         if command=='checkmail':
             subject='MKBACKUP: CONFIG OK %s ' % (job['name'], )
             sendmail(job['sender'], job['recipients'], subject, msg_body, [], job['smtp_host'], job['smtp_port'])
             
         if command in ('check', 'checkmail'):
             return
+
+        if typ=='none' or typ==None:
+            log.info('bye')
+            return
            
         start=int(time.time())
         process=subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        out, err=process.communicate()
+        std_out, std_err=process.communicate()
         end=int(time.time())
         
         logfile, last=None, 0
@@ -313,9 +329,36 @@ class NtBackup:
             if last<current:
                 logfile, last=full_filename, current
                 
-        print 'logfile', logfile
+        log.info('logfile=%s', logfile)
+        
+        ev_status, ev_out=ReadEvLog(NtBackup.ev_logtype, NtBackup.ev_source, start)
 
-        status, ev_out=ReadEvLog(NtBackup.ev_logtype, NtBackup.ev_source, start)
+        status='version=%s\n' % __version__
+        status='returncode=%r\n' % process.returncode
+        status+='start_epoch=%d\n' % start
+        status+='end_epoch=%d\n' % end
+        status+='start=%s\n' % time.ctime(start)
+        status+='end=%s\n' % time.ctime(end)
+        status+='cmdline=%s\n' % cmdline
+        status+='status=%s\n' % ev_status
+        
+        stat=os.stat(target)
+        status+='target_size=%d\n' % stat.st_size
+        status+='target_mtime_epoch=%d\n' % stat.st_mtime
+        status+='target_mtime=%s\n' % time.ctime(stat.st_mtime)
+        
+        #import ctypes
+        #free_bytes = ctypes.c_ulonglong(0)
+        #ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(os.path.dirname(target)), None, None, ctypes.pointer(free_bytes))
+        #print free_bytes.value
+
+        import win32file
+        free_bytes, total_bytes, total_free_bytes=win32file.GetDiskFreeSpaceEx(os.path.dirname(target))
+        status+='target_free_space=%d\n' % total_free_bytes
+
+        for line in status.split('\n'):
+            if line:
+                log.info('    %s', line)
         
         # HKEY_CURRENT_USER\Software\Microsoft\Ntbackup\Log Files
         
@@ -323,21 +366,25 @@ class NtBackup:
         # dir of target directory
         #
         args=[ os.environ['comspec'], '/c', 'dir', os.path.dirname(target) ]
-        print ' '.join(args)
         process=subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         dir_out, err=process.communicate()
         
         attachements=[ ('log.txt', logfile, None),
                        ('dir.txt', None, dir_out),
                        ('evlog.txt', None, ev_out),
-                       ('config.ini', job['config'], None)
+                       ('config.ini', job['config'], None),
+                       ('status.txt', None, status),
                     ]
         if job['selection'].startswith('@'):
             attachements.append(('selection.bks', job['selection'][1:], None))
+        if std_out:
+            attachements.append(('stdout.txt', None, std_out))
+        if std_err:
+            attachements.append(('stderr.txt', None, std_err))
         
-        subject='MKBACKUP BACKUP %s %s' % (status, job['name'])
+        subject='MKBACKUP BACKUP %s %s' % (ev_status, job['name'])
         sendmail(job['sender'], job['recipients'], subject, cmdline, attachements, job['smtp_host'], job['smtp_port'])
-    
+        
 
 archivers=dict((x.name, x) for x in (NtBackup, ))
 
@@ -388,7 +435,7 @@ def ReadEvLog(logtype, source, start, end=None):
                 cont=False
                 break
             if cont=='first':
-                print 'first %r %r %r %r' % (ev_obj.EventID=='8001', ev_obj.EventType==win32con.EVENTLOG_INFORMATION_TYPE, ev_obj.EventID, ev_obj.EventType)
+                #print 'first %r %r %r %r' % (ev_obj.EventID=='8001', ev_obj.EventType==win32con.EVENTLOG_INFORMATION_TYPE, ev_obj.EventID, ev_obj.EventType)
                 cont=True
                 if ev_obj.EventID==8019 and ev_obj.EventType==win32con.EVENTLOG_INFORMATION_TYPE:
                     status='OK'
@@ -435,8 +482,8 @@ class Destinations:
                 typ, period=selector.split('=', 1)
                 
                 try:
-                    if typ.lower()=='exempted':
-                        typ='exempted'
+                    if typ.lower()=='none':
+                        typ='none'
                     else:
                         typ=archiver.types[typ.lower()]
                 except KeyError:
@@ -506,9 +553,32 @@ parser.set_usage=('%prog [options] command [job..]\n'+
 parser.add_option("-c", "--config", dest="config", default='mkbackup.ini', help="use another ini file", metavar="config.ini")
 parser.add_option("-d", "--debug", dest="debug", action="store_true", default=False, help="switch to debug level")
 parser.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="write log to the terminal")
+parser.add_option("-l", "--logfile", dest="logfile", default='mkbackup.log', help="log to this file", metavar="mkbackup.log")
 
 cmd_options, cmd_args=parser.parse_args(sys.argv)
 
+print 'enable logging in %s' % cmd_options.logfile
+root_handler=logging.handlers.RotatingFileHandler(cmd_options.logfile, 'a', 1024**2, 2)
+root_handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S'))
+log=logging.getLogger()
+log.setLevel(logging.INFO)
+log.addHandler(root_handler)
+
+if cmd_options.debug:
+    root_logger.setLevel(logging.DEBUG)
+else:
+    # this stop early the logging process of loglevel<=DEBUG and should 
+    # improve performance a little bit
+    logging.disable(logging.DEBUG)
+
+if cmd_options.verbose:
+   console=logging.StreamHandler()
+   console.setFormatter(logging.Formatter('%(asctime)s,%(msecs)03d %(levelname)-3.3s %(message)s', '%H:%M:%S'))
+   log.addHandler(console)
+
+print 'logging enabled'
+log.info('start %r', sys.argv)
+
 if len(cmd_args)<2:
     parser.error('no "command" set')
     
@@ -533,29 +603,29 @@ config=ConfigParser.RawConfigParser(config_default)
 try:
     config.readfp(open(cmd_options.config))
 except IOError:
-    print "error reading configuration file %s" % cmd_options.config
-    raise
+    log.exception('error reading configuration file %s', cmd_options.config)
+    sys.exit(1)
 
 check_for_update=config.get('DEFAULT', 'update_check')
 if boolean.get(check_for_update.lower(), None)!=False:
     up2date, msg=update_check()
     if up2date=='no':
-        print '!'*72,'\n'
-        print msg
-        print '!'*72
+        log.warning('!'*60)
+        for line in msg:
+            log.warning(line)
+        log.warning('!'*60)
 
 err=0
 for job_name in job_list:
     try:
         program=config.get(job_name, 'program')
     except ConfigParser.NoOptionError, e:
-        print 'no "program" set for job "%s"' % job_name
+        log.error('no "program" set for job "%s"',job_name)
         err+=1   
     else:
         if not program in archivers:
-            print 'program "%s" unknow in job "%s"' % (program, job_name)
+            log.error('program "%s" unknow in job "%s"', program, job_name)
             err+=1   
-
 if err>0:
     sys.exit(3)
 
@@ -568,10 +638,13 @@ for job_name in job_list:
     job['name']=job_name
     job['config']=cmd_options.config
 
-    print '%s job="%s" archiver="%s"' % (command, job_name, program)
-    archiver.run(command, job)
-
-
+    log.info('start command=%s job=%s archiver=%s', command, job_name, program)
+    try:
+        archiver.run(command, job, log)
+    except:
+        log.exception('running command=%s job=%s archiver=%s', command, job_name, program)
+    else:
+        log.info('end command=%s job=%s archiver=%s', command, job_name, program)
 
 
 #<full,first su>S:\monthly\data-full-month-${month%2}.bkf
